@@ -1,7 +1,7 @@
 /*---------------------------------------------------------------------------*
   C MODULE  cf_nameinfo_p.c
 
-  Copyright (c) 2003-2005, James H. Woodyatt
+  Copyright (c) 2003-2006, James H. Woodyatt
   All rights reserved.
 
   Redistribution and use in source and binary forms, with or without
@@ -38,6 +38,8 @@
 #include <stdlib.h>
 #include <stdio.h>
 
+#include <netinet/in.h>
+
 #define INVALID_ARGUMENT(S)     (invalid_argument("Cf_nameinfo." S))
 #define FAILWITH(S)             (failwith("Cf_nameinfo." S))
 
@@ -140,6 +142,21 @@ value cf_nameinfo_sockaddr_cons(const struct sockaddr* saPtr, size_t saLen)
     if (sxPtr) {
         sxPtr->sx_socklen = saLen;
         memcpy(&sxPtr->sx_sockaddr, saPtr, saLen);
+        
+        #if defined(IN6_IS_ADDR_LINKLOCAL)
+        /* a rude hack to work around common bugs on some BSD systems */
+        if (sxPtr->sx_sockaddr.sa_family == AF_INET6) {
+            struct sockaddr_in6* sin6Ptr =
+                (struct sockaddr_in6*) &sxPtr->sx_sockaddr;
+            struct in6_addr* in6Ptr = &sin6Ptr->sin6_addr;
+            if (IN6_IS_ADDR_LINKLOCAL(in6Ptr)) {
+                uint16_t* u16Ptr = (uint16_t*) in6Ptr->s6_addr;
+                if (sin6Ptr->sin6_scope_id == 0)
+                    sin6Ptr->sin6_scope_id = ntohs(u16Ptr[1]);
+                u16Ptr[1] = 0;
+            }
+        }
+        #endif
     }
     
     return sxVal;
@@ -152,6 +169,8 @@ static Cf_socket_domain_t cf_nameinfo_pf_unspec = {
 
 static value* cf_nameinfo_unresolved_exn = 0;
 static value cf_nameinfo_domain_val = Val_unit;
+static value cf_nameinfo_socktype_val = Val_unit;
+static value cf_nameinfo_protocol_val = Val_unit;
 static value cf_nameinfo_unspecified_val = Val_unit;
 static value cf_nameinfo_default_ai_flags_val = Val_unit;
 
@@ -164,6 +183,25 @@ CAMLprim value cf_nameinfo_domain(value unit)
     CAMLreturn(cf_nameinfo_domain_val);
 }
 
+/*---
+    external socktype_:
+        unit -> unit Cf_socket.socktype_t = "cf_nameinfo_socktype"
+  ---*/
+CAMLprim value cf_nameinfo_socktype(value unit)
+{
+    CAMLparam0();
+    CAMLreturn(cf_nameinfo_socktype_val);
+}
+
+/*---
+    external protocol_: unit -> Cf_socket.protocol_t = "cf_nameinfo_protocol"
+  ---*/
+CAMLprim value cf_nameinfo_protocol(value unit)
+{
+    CAMLparam0();
+    CAMLreturn(cf_nameinfo_protocol_val);
+}
+
 /*---
     external unspecified_: unit -> address_t = "cf_nameinfo_unspecified"
   ---*/
@@ -183,10 +221,43 @@ CAMLprim value cf_nameinfo_default_ai_flags(value unit)
     CAMLreturn(cf_nameinfo_default_ai_flags_val);
 }
 
+
+/*---
+    external is_specific_socktype:
+        unit Cf_socket.socktype_t -> 'st Cf_socket.socktype_t -> bool =
+        "cf_nameinfo_is_specific_socktype"
+  ---*/
+CAMLprim value cf_nameinfo_is_specific_socktype(value unitStVal, value xStVal)
+{
+    CAMLparam2(unitStVal, xStVal);
+    CAMLreturn(Val_bool(Nativeint_val(unitStVal) == Nativeint_val(xStVal)));
+}
+
+/*---
+    external is_specific_domain:
+        unit Cf_socket.sockaddr_t -> 'af Cf_socket.domain_t -> bool =
+        "cf_nameinfo_is_specific_domain"
+  ---*/
+CAMLprim value cf_nameinfo_is_specific_domain(value sxVal, value domainVal)
+{
+    CAMLparam2(sxVal, domainVal);    
+    const Cf_socket_sockaddrx_unit_t* sxPtr;
+    const Cf_socket_domain_t* domainPtr;
+    int result;
+    
+    sxPtr = Cf_socket_sockaddrx_unit_val(sxVal);
+    domainPtr = Cf_socket_domain_val(domainVal);
+    
+    result =
+        !!((int) domainPtr->d_family == (int) sxPtr->sx_sockaddr.sa_family);
+    
+    CAMLreturn(Val_bool(result));
+}
+
 /*---
     external specialize_sockaddr:
         unit Cf_socket.sockaddr_t -> 'af Cf_socket.domain_t ->
-        'af Cf_socket.sockaddr_t option = "cf_nameinfo_specialize_sockaddr"
+        'af Cf_socket.sockaddr_t = "cf_nameinfo_specialize_sockaddr"
   ---*/
 CAMLprim value cf_nameinfo_specialize_sockaddr(value sxVal, value domainVal)
 {
@@ -198,16 +269,12 @@ CAMLprim value cf_nameinfo_specialize_sockaddr(value sxVal, value domainVal)
     
     sxPtr = Cf_socket_sockaddrx_unit_val(sxVal);
     domainPtr = Cf_socket_domain_val(domainVal);
-    resultVal = Val_int(0);
-    
-    if ((int) domainPtr->d_family == (int) sxPtr->sx_sockaddr.sa_family) {
-        someVal = domainPtr->d_consaddr((struct sockaddr*) &sxPtr->sx_sockaddr,
-            domainPtr->d_socklen);
-
-        resultVal = alloc_small(1, 0);
-        Store_field(resultVal, 0, someVal);
-    }
     
+    if ((int) domainPtr->d_family != (int) sxPtr->sx_sockaddr.sa_family)
+        caml_raise_not_found();
+        
+    resultVal = domainPtr->d_consaddr((struct sockaddr*) &sxPtr->sx_sockaddr,
+        domainPtr->d_socklen);
     CAMLreturn(resultVal);
 }
 
@@ -343,7 +410,7 @@ static void cf_nameinfo_raise_unresolved
     resultVal = alloc_small(2, 0);
     Store_field(resultVal, 0, *cf_nameinfo_unresolved_exn);
     Store_field(resultVal, 1, unresolvedVal);
-    mlraise(resultVal);
+    caml_raise(resultVal);
     
     CAMLreturn0;
 }
@@ -422,23 +489,23 @@ CAMLprim value cf_nameinfo_of_address
         | A_servicename of string
         | A_bothnames of string * string
     
-    type addrinfo_t = {
+    type ('af, 'st) addrinfo = private {
         ai_flags: to_address_flags_t;
-        ai_family: unit Cf_socket.domain_t;
-        ai_socktype: unit Cf_socket.socktype_t;
+        ai_family: 'af Cf_socket.domain_t;
+        ai_socktype: 'st Cf_socket.socktype_t;
         ai_protocol: Cf_socket.protocol_t;
         ai_cname: string option;
-        ai_addr: AF.address_t;
+        ai_addr: 'af Cf_socket.sockaddr_t;
     }
     
     external to_address:
-        ?hint:addrinfo_t -> to_address_arg_t -> addrinfo_t list =
-        "cf_nameinfo_to_address"
+        ('af, 'st) addrinfo -> to_address_arg_t -> ('af, 'st) addrinfo list =
+        "cf_nameinfo_to_addresses"
   ---*/
-CAMLprim value cf_nameinfo_to_address(value hintOptVal, value argVal)
+CAMLprim value cf_nameinfo_to_addresses(value hintVal, value argVal)
 {
-    CAMLparam2(hintOptVal, argVal);
-    CAMLlocal4(listVal, hintVal, flagsVal, prevVal);
+    CAMLparam2(hintVal, argVal);
+    CAMLlocal3(listVal, flagsVal, prevVal);
     CAMLlocal4(nextVal, infoVal, familyVal, socktypeVal);
     CAMLlocal4(protocolVal, cnameVal, addrVal, strVal);            
     
@@ -471,20 +538,23 @@ CAMLprim value cf_nameinfo_to_address(value hintOptVal, value argVal)
         break;
     }
     
-    if (Is_block(hintOptVal)) {        
-        memset(&hints, 0, sizeof hints);
-        hintVal = Field(hintOptVal, 0);
-        flagsVal = Field(hintVal, 0);
-        if (Bool_val(Field(flagsVal, 0))) hints.ai_flags |= AI_PASSIVE;
-        if (Bool_val(Field(flagsVal, 1))) hints.ai_flags |= AI_CANONNAME;
-        if (Bool_val(Field(flagsVal, 2))) hints.ai_flags |= AI_NUMERICHOST;
-        
-        hints.ai_family = Nativeint_val(Field(hintVal, 1));
-        hints.ai_socktype = Nativeint_val(Field(hintVal, 2));
-        hints.ai_protocol = Nativeint_val(Field(hintVal, 3));
+    memset(&hints, 0, sizeof hints);
+    flagsVal = Field(hintVal, 0);
+    if (Bool_val(Field(flagsVal, 0))) hints.ai_flags |= AI_PASSIVE;
+    if (Bool_val(Field(flagsVal, 1))) hints.ai_flags |= AI_CANONNAME;
+    if (Bool_val(Field(flagsVal, 2))) hints.ai_flags |= AI_NUMERICHOST;
+    
+    hints.ai_family = Nativeint_val(Field(hintVal, 1));
+    hints.ai_socktype = Nativeint_val(Field(hintVal, 2));
+    hints.ai_protocol = Nativeint_val(Field(hintVal, 3));
 
+    if (
+        hints.ai_family != PF_UNSPEC ||
+        hints.ai_socktype != 0 ||
+        hints.ai_protocol != 0 ||
+        hints.ai_flags
+       )
         hintsPtr = &hints;
-    }
         
     enter_blocking_section();
     error = getaddrinfo(hostname, servname, hintsPtr, &resultPtr);
@@ -567,6 +637,12 @@ CAMLprim value cf_nameinfo_init(value unit)
     register_global_root(&cf_nameinfo_domain_val);
     cf_nameinfo_domain_val =
         cf_socket_domain_alloc(&cf_nameinfo_pf_unspec);
+
+    register_global_root(&cf_nameinfo_socktype_val);
+    cf_nameinfo_socktype_val = caml_copy_nativeint(0);
+
+    register_global_root(&cf_nameinfo_protocol_val);
+    cf_nameinfo_protocol_val = caml_copy_nativeint(0);
     
     register_global_root(&cf_nameinfo_unspecified_val);
     saPtr = (struct sockaddr*) &ss;
