@@ -1,7 +1,7 @@
 (*---------------------------------------------------------------------------*
   INTERFACE  cf_uri.mli
 
-  Copyright (c) 2003-2005, James H. Woodyatt
+  Copyright (c) 2003-2006, James H. Woodyatt
   All rights reserved.
 
   Redistribution and use in source and binary forms, with or without
@@ -41,74 +41,74 @@
 (** {6 Types} *)
 
 (** The sum type of "host" components of hierarchical URI's. *)
-type host_t =
+type host =
     | H_hostname of string
     | H_ip4_addr of Cf_ip4_addr.opaque Cf_ip4_addr.t
 
 (** The type of "server" components of hierarchical URI's. *)
-type server_t = {
+type server = {
     srv_user: string option;    (** An optional user name. *)
-    srv_host: host_t;           (** The hostname or IPv4 address. *)
+    srv_host: host;             (** The hostname or IPv4 address. *)
     srv_port: int option;       (** An optional port number. *)
 }
 
 (** The type of "authority" components of hierarchical URI's. *)
-type authority_t =
-    | A_server of server_t option   (** Server-based naming authority. *)
+type authority =
+    | A_server of server option     (** Server-based naming authority. *)
     | A_reg_name of string          (** Registry-based naming authority. *)
 
 (** The type of path segments in hierarchical URI's. *)
-type segment_t = {
-    seg_name: string;           (** Segment name. *)
-    seg_params: string list;    (** Segment parameters list. *)
+type segment = {
+    seg_name: string;               (** Segment name. *)
+    seg_params: string list;        (** Segment parameters list. *)
 }
 
 (** The type of network paths in hierarchical URI's. *)
-type net_path_t = {
-    net_authority: authority_t;     (** Naming authority. *)
-    net_path: segment_t list;       (** List of path segments. *)
+type net_path = {
+    net_authority: authority;       (** Naming authority. *)
+    net_path: segment list;         (** List of path segments. *)
 }
 
 (** The type of hierarchical URI network paths. *)
-type net_path_v = [ `Net of net_path_t ]
+type net = [ `Net of net_path ]
 
 (** The type of hierarchical URI absolute paths. *)
-type abs_path_v = [ `Abs of segment_t * segment_t list ]
+type abs = [ `Abs of segment * segment list ]
 
 (** The type of hierarchical URI relative paths. *)
-type rel_path_v = [ `Rel of segment_t list ]
+type rel = [ `Rel of segment list ]
 
 (** The sum type of hierarchical URI paths. *)
-type path_t = [ net_path_v | abs_path_v | rel_path_v ]
+type path = [ net | abs | rel ]
 
 (** The type of absolute hierarchical URI paths with optional query parts. *)
-type abs_special_hier_t = {
-    abs_hier_path: [ net_path_v | abs_path_v ];     (** Path. *)
-    abs_hier_query: string option;                  (** Query part. *)
+type abs_special_hier = {
+    abs_hier_path: [ net | abs ];       (** Path. *)
+    abs_hier_query: string option;      (** Query part. *)
 }
 
 (** The sum type of absolute URI parts. *)
-type abs_special_t =
-    | S_hier of abs_special_hier_t      (** Hierarchical URI part. *)
+type abs_special =
+    | S_hier of abs_special_hier        (** Hierarchical URI part. *)
     | S_opaque of string                (** Opaque URI part. *)
 
 (** The type of absolute URI. *)
-type absolute_t = {
+type absolute = {
     abs_scheme: string;             (** URI scheme name. *)
-    abs_special: abs_special_t;     (** Absolute URI part. *)
+    abs_special: abs_special;       (** Absolute URI part. *)
 }
 
 (** The type of relative URI. *)
-type relative_t = {
-    rel_path: path_t;               (** Path. *)
+type relative = {
+    rel_path: path;                 (** Path. *)
     rel_query: string option;       (** Query part. *)
 }
 
 (** The type of URI. *)
-type t = A of absolute_t | R of relative_t
+type t = A of absolute | R of relative
 
 (** The type of URI references. *)
-type reference_t = {
+type reference = {
     ref_uri: t;                     (** URI *)
     ref_fragment: string option;    (** Optional fragment suffix. *)
 }
@@ -137,7 +137,7 @@ val unescape: string -> string
     [Invalid_argument] if the base URI is opaque, and raises [Rel_undefined]
     if the URI cannot be referred, i.e. too many ".." segments.
 *)
-val refer_to_base: base:absolute_t -> rel:relative_t -> absolute_t
+val refer_to_base: base:absolute -> rel:relative -> absolute
 
 (** Use [message_to_uri m] to create a URI by parsing the contents of the
     message [m].  Raises [Invalid_argument] if the message does not contain a
@@ -152,13 +152,13 @@ val message_to_uri: Cf_message.t -> t
     [Rel_undefined] if the message contains a relative URI that cannot be
     referred by the base URI.
 *)
-val message_to_absolute_uri: base:absolute_t -> Cf_message.t -> absolute_t
+val message_to_absolute_uri: base:absolute -> Cf_message.t -> absolute
 
 (** Use [message_to_uri_reference m] to create a URI reference by parsing the
     contents of the message [m].  Raises [Invalid_argument] if the message does
     not contain a valid URI reference.
 *)
-val message_to_uri_reference: Cf_message.t -> reference_t
+val message_to_uri_reference: Cf_message.t -> reference
 
 (** Use [message_to_absolute_uri_reference ~base m] to create a URI reference
     to an absolute URI by parsing the contents of the message [m] and using
@@ -168,37 +168,37 @@ val message_to_uri_reference: Cf_message.t -> reference_t
     URI that cannot be referred by the base URI.
 *)
 val message_to_absolute_uri_reference:
-    base:absolute_t -> Cf_message.t -> reference_t
+    base:absolute -> Cf_message.t -> reference
 
 (** Use [emit_host pp host] to print the host part of a URI [host] with the
     formatter [pp].  Reserved characters in the host name are escaped in the
     output.
 *)
-val emit_host: Format.formatter -> host_t -> unit
+val emit_host: Format.formatter -> host -> unit
 
 (** Use [emit_server pp server] to print the server part of a URI [server] with
     the formatter [pp].  Reserved characters in the host name or user name are
     escaped in the output.
 *)
-val emit_server: Format.formatter -> server_t -> unit
+val emit_server: Format.formatter -> server -> unit
 
 (** Use [emit_authority pp auth] to print the name authority part of a URI
     [authority] with the formatter [pp].  Reserved characters are escaped
     appropriately.
 *)
-val emit_authority: Format.formatter -> authority_t -> unit
+val emit_authority: Format.formatter -> authority -> unit
 
 (** Use [emit_path pp path] to print the path component of a URI [path] with
     the formatter [pp].  Reserved characters in the path segments or path
     segment parameter lists are escaped in the output.
 *)
-val emit_path: Format.formatter -> [< path_t ] -> unit
+val emit_path: Format.formatter -> [< path ] -> unit
 
 (** Use [emit_abs_special pp abs] to print the absolute URI specialization
     [abs] with the formatter [pp].  All reserved characters are escaped
     appropriately in the output.
 *)
-val emit_abs_special: Format.formatter -> abs_special_t -> unit
+val emit_abs_special: Format.formatter -> abs_special -> unit
 
 (** Use [emit_uri pp uri] to print the URI [uri] with the formatter [pp].  All
     reserved characters are escaped appropriately in the output.
@@ -209,7 +209,7 @@ val emit_uri: Format.formatter -> t -> unit
     the formatter [pp].  All reserved characters are escaped appropriately in
     the output.
 *)
-val emit_uri_reference: Format.formatter -> reference_t -> unit
+val emit_uri_reference: Format.formatter -> reference -> unit
 
 (** Use [message_of_uri uri] to produce a message containing the formatted URI
     string produced by emitting [uri] into a string.
@@ -219,6 +219,6 @@ val message_of_uri: t -> Cf_message.t
 (** Use [message_of_uri_reference uriref] to produce a message containing the
     formatted URI reference string produced by emitting [uriref] into a string.
 *)
-val message_of_uri_reference: reference_t -> Cf_message.t
+val message_of_uri_reference: reference -> Cf_message.t
 
 (*--- End of File [ cf_uri.mli ] ---*)
