@@ -1,7 +1,7 @@
 (*---------------------------------------------------------------------------*
   IMPLEMENTATION  t_cf.ml
 
-  Copyright (c) 2003-2005, James H. Woodyatt
+  Copyright (c) 2003-2006, James H. Woodyatt
   All rights reserved.
 
   Redistribution and use in source and binary forms, with or without
@@ -32,10 +32,8 @@
 
 Random.self_init ();;
 
-(**)
-let jout = Cf_journal.stdout;;
-jout#setlimit `None;;
-(**)
+let jout = Cf_journal.stdout
+let _ = jout#setlimit `Notice
 
 (*
 Gc.set {
@@ -58,13 +56,13 @@ module T1 = struct
     let decreasing = to_seq_decr
     
     let rec printlist s nl =
-        (*
-        Printf.printf "%s: [" s;
-        List.iter (fun n -> Printf.printf " %d;" n) nl;
-        Printf.printf " ]\n";
-        flush stdout;
-        *)
-        ()
+        assert begin
+            let b = Buffer.create 40 in
+            Printf.bprintf b "%s: [" s;
+            List.iter (fun n -> Printf.bprintf b " %d;" n) nl;
+            Printf.bprintf b " ]\n";
+            jout#debug "%s" (Buffer.contents b)
+        end
     
     let to_list tree =
         Cf_seq.fold (fun v (e, _) -> e :: v) [] (decreasing tree)
@@ -82,9 +80,11 @@ module T1 = struct
         printlist msg (to_list tree);
         try
             let e = search key tree in
+            assert (member key tree);
             if (e / 2) <> key then failwith "testfind key <> e / 2"
         with
         | Not_found ->
+            assert (not (member key tree));
             failwith "testfind -> Not_found"
     
     let testextract tree key =
@@ -155,15 +155,14 @@ module T1 = struct
     let test () = test0 ()
 end
 
-module T2 = struct
-    (* open Cf_rbset *)
-        
+module T2 = struct        
     module S1 = Set.Make(Cf_ordered.Int_order)
     module S2 = Cf_rbtree.Set(Cf_ordered.Int_order)
     
     let bound = 64
     let iterations = 512
     
+    (*
     let print_list =
         let rec loop = function
             | [] -> ()
@@ -175,6 +174,7 @@ module T2 = struct
             loop r;
             print_string " ]\n";
             flush stdout
+    *)
     
     let rec loop n s1 s2 =
         let x = Random.int bound in
@@ -203,7 +203,8 @@ module T3 = struct
     let test1 () =
         let m = M.replace (0, "zero") M.nil in
         let s = M.search 0 m in
-        assert (s = "zero")
+        assert (s = "zero");
+        assert (M.member 0 m)
     
     let nearest_succ key m =
         match Lazy.force (M.nearest_incr key m) with
@@ -257,25 +258,19 @@ module T4 = struct
     (* open Cf_kotdeque *)
     open Cf_deque
     
-    (**)
-    let aux msg i q =
-        (*
-        Printf.printf "%s i=%d q=%s\n" msg i (sprint string_of_int q);
-        flush stdout
-        *)
-        (**)
-        Printf.printf "%s i=%d\n" msg i;
-        flush stdout
-        (**)
+    let aux msg i _ (* q *) =
+        assert begin
+            (* jout#debug "%s i=%d q=%s\n" msg i (sprint string_of_int q) *)
+            (**) jout#debug "%s i=%d\n" msg i (**)
+        end
         
     let modaux msg modulo i q =
         if ((i / modulo) * modulo) = i then aux msg i q
-    (**)
 
     let test1 pop push =
-        let n = 1024 and m = 128 in
+        let n = 1024 in
         let rec loop1 i q =
-            (* aux "+" i q; *)
+            aux "+" i q;
             if i < n then loop1 (succ i) (push i q) else loop2 0 q
         and loop2 i q =
             (* aux "-" i q; *)
@@ -285,7 +280,7 @@ module T4 = struct
                     failwith "[1] pop: None"
                 | Some (j, q) ->
                     if i <> j then
-                        failwith (Printf.sprintf "[1] i <> j j=%d" j);
+                        jout#fail "[1] i <> j j=%d" j;
                     loop2 (succ i) q
             end
             else
@@ -387,82 +382,6 @@ module T5 = struct
 end
 
 module T6 = struct
-    open Printf
-    open Cf_lexer.Op
-    open Cf_parser.Op
-        
-    module L1 = struct
-        let cursor_ = new Cf_lexer.line_cursor "\n"
-        
-        let test1 lexer =
-            let token0 = "abaabaababbabb" in
-            let input = token0 (* ^ "jhw" *) in
-            let s = Cf_seq.of_substring input 0 in
-            let s = Cf_parser.X.weave cursor_ s in
-            let token, s' =
-                match lexer s with
-                | Some x -> x
-                | None -> failwith "No match!"
-            in
-            if Lazy.force s' <> Cf_seq.Z then
-                failwith "Unexpected trailer [1]!";
-            if token <> token0 then
-                failwith (sprintf "Bad match! [msg='%s']" token)
-        
-        let test2 lexer =
-            let token0 = "abaabaababbabb" in
-            let input = token0 ^ "jhw" in
-            let s = Cf_seq.of_substring input 0 in
-            let s = Cf_parser.X.weave cursor_ s in
-            let token, s' =
-                match lexer s with
-                | Some x -> x
-                | None -> failwith "No match!"
-            in
-            if Lazy.force s' = Cf_seq.Z then
-                failwith "Expected trailer [2]!";
-            if token <> token0 then
-                failwith (sprintf "Bad match! [msg='%s']" token)
-    end
-    
-    module L2 = struct
-        let p =
-            let q0 = Cf_scan_parser.scanfx "%3u" (fun y -> y) in
-            let q1 = Cf_scan_parser.scanfx "%c" (fun y -> y) in
-            let q2 = Cf_scan_parser.scanfx "%3u" (fun y -> y) in
-            q0 >>= fun v0 ->
-            q1 >>= fun v1 ->
-            q2 >>= fun v2 ->
-            ~:(v0, v1, v2)
-        
-        let c = new Cf_lexer.cursor 0
-        
-        let y s = p (Cf_parser.X.weave ~c (Cf_seq.of_string s))
-        
-        let test () =
-            match y "1234567" with
-            | Some ((123, '4', 567), z) when Lazy.force z = Cf_seq.Z ->
-                ()
-            | Some ((123, '4', 567), z) ->
-                failwith (Cf_seq.to_string (Cf_seq.first z))
-            | Some ((v0, v1, v2), _) ->
-                failwith (Printf.sprintf "%02u, '%c', %02u" v0 v1 v2)
-            | _ ->
-                failwith "No match!"
-    end
-
-    let test () =
-        let lexer =
-            Cf_lexer.create begin
-                (!*(!:'a' $| !:'b')) $& !$"abb" $^ (fun x -> x)
-            end
-        in
-        L1.test1 lexer;
-        L1.test2 lexer;
-        L2.test ()
-end
-
-module T7 = struct
     open Printf
     open Cf_lex.Op
     open Cf_parser.Op
@@ -622,7 +541,7 @@ module T7 = struct
         L5.test ()
 end
 
-module T8 = struct
+module T7 = struct
     open Cf_parser.Op
     
     let digit =
@@ -759,7 +678,7 @@ module T8 = struct
         List.iter t_pos positive
 end
 
-module T9 = struct
+module T8 = struct
     module L1 = struct
         module C = Cf_cmonad
         open C.Op
@@ -881,7 +800,7 @@ module T9 = struct
         assert true
 end
 
-module T10 = struct
+module T9_old = struct
     (*
     let rec logflow w =
         match Lazy.force w with
@@ -911,7 +830,7 @@ module T10 = struct
     *)
 
     open Cf_cmonad.Op
-    open Cf_gadget
+    open Cf_state_gadget
             
     let rec divider i o =
         let i = (i :> ('a, int, float) rx) in
@@ -978,7 +897,105 @@ module T10 = struct
         if not (Cf_seq.equal x o) then failwith "Transform failed!"
 end
 
-module T11 = struct
+module T9 = struct
+    (*
+    let rec logflow w =
+        match Lazy.force w with
+        | Cf_flow.Z ->
+            print_string "| Z\n";
+            flush stdout;
+            Cf_flow.Z
+        | Cf_flow.P (hd, tl) ->
+            (* if (int_of_float hd) mod 1000 = 0 then begin *)
+                Printf.printf "| P %f\n" hd;
+                flush stdout;
+            (* end; *)
+            Cf_flow.P (hd, lazy (logflow tl))
+        | Cf_flow.Q f ->
+            Cf_flow.Q begin fun i ->
+                Printf.printf "| Q %d\n" i;
+                flush stdout;
+                logflow (lazy (f i))
+            end
+    
+    let rec circuit0 =
+        Cf_flow.Q begin fun n ->
+            let hd = (float_of_int n) /. 2.0 in
+            let tl = Lazy.lazy_from_val circuit0 in
+            Cf_flow.P (hd, tl)
+        end
+    *)
+
+    open Cf_cmonad.Op
+    open Cf_gadget
+        
+    class divider i o =
+        let i = (i :> ('a, int, float) rx) in
+        let o = (o :> ('b, int, float) tx) in
+        object(self)
+            inherit [int, float] start
+            inherit [int, float] next
+            
+            val count = 0
+            
+            method private input (`I n) =
+                o#put (`O ((float_of_int n) /. 2.0)) >>= fun () ->
+                {< count = succ count >}#next
+            
+            method private guard = i#get self#input
+        end
+    
+    let ingest isrc =
+        let isrc = (isrc :> ('a, int, float) tx) in
+        let rec loop () =
+            read >>= fun i ->
+            isrc#put (`I i) >>= loop
+        in
+        loop ()
+    
+    let render osnk =
+        let osnk = (osnk :> ('a, int, float) rx) in
+        let rec loop () =
+            guard begin
+                osnk#get begin fun (`O n) ->
+                    write n >>= loop
+                end
+            end
+        in
+        loop ()
+    
+    let gadget () =
+        simplex >>= fun (efRx, efTx) ->
+        simplex >>= fun (enRx, enTx) ->
+        start (ingest enTx) >>= fun () ->
+        start (render efRx) >>= fun () ->
+        let m = new divider enRx efTx in
+        m#start
+    
+    let input =
+        let rec loop n = Cf_seq.P (n, lazy (loop (succ n))) in
+        fun () -> lazy (loop 1)
+    
+    let output =
+        let rec loop n =
+            let x = (float_of_int n) /. 2.0 in
+            Cf_seq.P (x, lazy (loop (succ n)))
+        in
+        fun () -> lazy (loop 1)
+    
+    open Cf_flow.Op
+    
+    let test () =
+        let n = 100 in
+        let i = Cf_seq.limit n (input ()) in
+        let o = Cf_seq.limit n (output ()) in
+        let divflow = eval (gadget ()) in
+        (* let divflow = lazy (logflow divflow) in *)
+        let x = Cf_flow.to_seq (Cf_flow.of_seq i -=- divflow) in
+        if not (Cf_seq.equal x o) then failwith "Transform failed!"
+end
+
+module T10 = struct
     module V6 = Cf_ip6_addr
     open Printf
     
@@ -1008,7 +1025,7 @@ module T11 = struct
         v6_unicast_test ()
 end
 
-module T12 = struct
+module T11 = struct
     let len = 100
 
     let test () =
@@ -1053,7 +1070,7 @@ module T12 = struct
                 (Unix.error_message e) fn)
 end
 
-module T13 = struct
+module T12 = struct
     let len = 100
 
     let test () =
@@ -1105,7 +1122,7 @@ module T13 = struct
                 (Unix.error_message e) fn)
 end
 
-module T14 = struct
+module T13 = struct
     open Cf_uri
     
     module L1 = struct
@@ -1214,7 +1231,7 @@ module T14 = struct
         L2.test ();
 end
 
-module T15 = struct
+module T14 = struct
 (*
 3fffe33e1b840309 365  Sun -1000000-12-31 23:59:59 +0000
 3fffe33e1b84030a 000  Mon -999999-01-01 00:00:00 +0000
@@ -1351,35 +1368,36 @@ module T15 = struct
     let test () = List.iter L1.y L1.x
 end
 
-module T16 = struct
+module T15 = struct
     open Cf_poll
     
     class clock n dt =
         object
-            inherit [unit Cf_poll.state_t] time Cf_tai64n.last as super
+            inherit [unit] time dt as super
             
-            val mutable count_: int = n
-            
-            method private load_ p =
-                epoch_ <- Cf_tai64n.add (Cf_tai64n.now ()) dt;
-                super#load_ p
-            
-            method private service p =
+            val mutable count_ = n
+                        
+            method private service _ =
+                let tai, frac = Cf_tai64n.decompose epoch_ in
+                let utc = Cf_stdtime.utc_of_tai64 tai in
+                jout#info
+                    "T16.clock#service: %04u-%02u-%02u %02u:%02u:%02u.%06u "
+                    utc.Cf_stdtime.year utc.Cf_stdtime.month utc.Cf_stdtime.day
+                    utc.Cf_stdtime.hour utc.Cf_stdtime.minute
+                    utc.Cf_stdtime.second frac;
                 if count_ > 0 then begin
                     count_ <- pred count_;
-                    epoch_ <- Cf_tai64n.add epoch_ dt;
-                    super#load_ p;
                     state_
                 end
                 else
-                    `Final ()
+                    Cf_poll.Final ()
             
             method count = count_
         end
     
     class interrupt n dt =
         object(self:'self)
-            inherit [int Cf_poll.state_t] signal Sys.sigint as super
+            inherit [int] signal Sys.sigint as super
 
             val clock_ = new clock n dt
             
@@ -1391,9 +1409,9 @@ module T16 = struct
                 super#unload_ p;
                 clock_#unload
             
-            method private service p =
+            method private service _ =
                 clock_#unload;
-                `Final clock_#count
+                Cf_poll.Final clock_#count
             
             method canget =
                 if not super#canget then begin
@@ -1401,7 +1419,7 @@ module T16 = struct
                         false
                     else begin
                         self#unload;
-                        state_ <- `Final 0;
+                        state_ <- Cf_poll.Final 0;
                         true
                     end
                 end
@@ -1426,7 +1444,7 @@ module T16 = struct
         ignore (Unix.sigprocmask Unix.SIG_SETMASK save)
 end
 
-module T17 = struct
+module T16 = struct
     open Cf_scmonad.Op
     
     module String_set = Cf_rbtree.Set(String)
@@ -1459,6 +1477,51 @@ module T17 = struct
         if s2 <> s2' then failwith "Error in uniq!"
 end
 
+module T17 = struct
+    open Cf_state_gadget
+    open Cf_cmonad.Op
+    
+    let gadget limit =
+        read >>= fun () ->
+        duplex >>= fun ((inRx, outTx), (outRx, inTx)) ->
+        let inRx = (inRx :> (unit, unit, unit) rx) in
+        let outTx = (outTx :> (unit, unit, unit) tx) in
+        let outRx = (outRx :> (unit, unit, unit) rx) in
+        let inTx = (inTx :> (unit, unit, unit) tx) in
+        inTx#put () >>= fun () ->
+        wrap inRx outTx Cf_flow.nop >>= fun () ->
+        let rec loop () =
+            guard begin
+                outRx#get begin fun () ->
+                    load >>= fun n ->
+                    if n < limit then begin
+                        (*
+                        if n mod 1000 = 0 then begin
+                            Printf.printf "n=%08u\n" n;
+                            flush stdout
+                        end;
+                        *)
+                        store (succ n) >>= fun () ->
+                        inTx#put () >>= loop
+                    end
+                    else
+                        Cf_cmonad.nil 
+                end
+            end
+        in
+        start (loop ()) 0
+    
+    let test () =
+        let limit = 10000 in
+        let gc = Gc.get () in
+        Gc.set { gc with Gc.stack_limit = 4 * 0x400 };
+        let w = eval (gadget limit) () in
+        match Lazy.force w with
+        | Cf_flow.Q f -> let _ = f () in Gc.set gc
+        | _ -> failwith "Cf_state_gadget not evaluated to Cf_flow.Q state."
+end
+
+(*
 module T18 = struct
     open Cf_gadget
     open Cf_cmonad.Op
@@ -1487,7 +1550,7 @@ module T18 = struct
                         inTx#put () >>= loop
                     end
                     else
-                        Cf_cmonad.return ()
+                        Cf_cmonad.nil 
                 end
             end
         in
@@ -1500,15 +1563,16 @@ module T18 = struct
         let w = eval (gadget limit) () in
         match Lazy.force w with
         | Cf_flow.Q f -> let _ = f () in Gc.set gc
-        | _ -> failwith "Cf_gadget not evaluated to Cf_flow.Q state."
+        | _ -> failwith "Cf_state_gadget not evaluated to Cf_flow.Q state."
 end
+*)
 
 let main () =
     let tests = [
         T1.test; T2.test; T3.test; T4.test; T5.test;
         T6.test; T7.test; T8.test; T9.test; T10.test;
         T11.test; T12.test; T13.test; T14.test; T15.test;
-        T16.test; T17.test; T18.test
+        T16.test; T17.test (*; T18.test *)
     ] in
     Printf.printf "1..%d\n" (List.length tests);
     flush stdout;
